<template>
  <el-container class="is-vertical total-layout">
    <template v-if="menuStyle == 'sideMenu'">
      <Navigator :menuShow.sync="menuShow" :showTopMenu="true" />
      <el-container class="body-container">
        <el-main
          class="main-container"
          :class="$route.meta.isHome ? 'home-page' : ''"
        >
          <transition :name="transitionName" mode="out-in" appear>
            <template v-if="showPage">
              <router-view
                class="fullheight"
                @menuChange="handleMenuChange"
              ></router-view>
            </template>
            <template v-else>
              <router-view
                class="fullheight"
                @menuChange="handleMenuChange"
              ></router-view>
            </template>
            <!-- <el-card
                            class="app-box__card"
                            :class="{
                                'box-card-noheader':
                                    $route.meta.isHome || !showHeader,
                                'single-sepical':
                                    $route.path == '/sysplat/tenantInfoManager',
                            }"
                            shadow="never"
                            v-if="showPage"
                            :key="$route.path"
                        >
                            <keep-alive v-if="!$route.meta.keepAlive">
                                <router-view
                                    class="fullheight"
                                    @menuChange="handleMenuChange"
                                ></router-view>
                            </keep-alive>
                            <router-view
                                v-else
                                class="fullheight"
                                @menuChange="handleMenuChange"
                            ></router-view>
                        </el-card>
                        <el-card
                            class="app-box__card"
                            :class="{
                                'box-card-noheader': !showHeader,
                                'single-sepical':
                                    $route.path == '/sysplat/tenantInfoManager',
                            }"
                            shadow="never"
                            :key="$route.path"
                            v-else
                        >
                            <keep-alive v-if="!$route.meta.keepAlive">
                                <router-view
                                    class="fullheight"
                                    @menuChange="handleMenuChange"
                                ></router-view>
                            </keep-alive>
                            <router-view
                                v-else
                                class="fullheight"
                                @menuChange="handleMenuChange"
                            ></router-view>
                        </el-card> -->
          </transition>
        </el-main>
      </el-container>
    </template>
    <!-- 多模式 -->
    <!-- <side-menu
            @menuChange="handleMenuChange"
            :menuShow.sync="menuShow"
        /> -->
    <!-- <template v-if="menuStyle == 'topMenu'">
            <Navigator :menuShow.sync="menuShow" :showTopMenu="true" />
            <router-view />
        </template>
        <template v-if="menuStyle == 'topSide'">
            <Navigator :menuShow.sync="menuShow" :showTopMenu="true" />
            <router-view />
        </template> -->
  </el-container>
</template>

<script>
import Navigator from './Navigator.vue'
import SideMenu from './SideMenu.vue'
import menu from '@/api/menu.js'
import portal from '@/api/portal.js'
import pako from 'pako'
export default {
  components: {
    Navigator,
    SideMenu
  },
  data() {
    return {
      menuShow: true,
      currentMenuLabel: '',
      showHeader: true,
      menuStyle: 'sideMenu', // sideMenu  topMenu  topSide
      transitionName: 'fold-left',
      showPage: true,
      isCurrentWindow: true
    }
  },
  watch: {
    $route(to, from) {
      if (!to.hash) {
        this.showPage = !this.showPage
        const toDepth = to.path.split('/').length
        const fromDepth = from.path.split('/').length
        this.transitionName = toDepth < fromDepth ? 'fold-right' : 'fold-left'
      }
    }
  },
  created() {
    this.heartCheckFun()
    this.initWebSocket()
  },
  mounted() {
    document.addEventListener('visibilitychange', this.windowVisiable)
    window.onresize = () => {
      this.$root.$emit('resize')
    }
    menu.getMenuByAlias(this.$route.name, m => {
      if (m) {
        this.$emit('menuChange', m)
      }
    })
    this.$bus.$on('login', () => {
      this.heartCheckFun()
    })
    // 发送消息
    // this.$bus.$on('sendWebsock',()=>{
    //     this.websocketSend("{'messageType':'topic','message': '发送'}")
    // })
  },
  beforeDestroy() {
    this.isCurrentWindow = false
    document.removeEventListener('visibilitychange', this.windowVisiable)
  },
  methods: {
    windowVisiable(e) {
      if (e.target.visibilityState == 'visible') {
        // console.log('回到当前tab标签')
        this.isCurrentWindow = true
      } else {
        // console.log('离开当前tab标签')
        this.isCurrentWindow = false
      }
    },
    handleMenuChange(menu) {
      this.currentMenuLabel = menu.name
      this.showHeader = !(menu.children && menu.children.length > 0)
    },
    initWebSocket() {
      // WebSocket与普通的请求所用协议有所不同，ws等同于http，wss等同于https
      const token = this.$store.state.login.currentUser.token
      const url =
        window.context.WebSocket.replace('https://', 'wss://').replace(
          'http://',
          'ws://'
        ) + '/websocket'
      // ?token=' +
      // token
      console.log(url, 'url')
      this.websock = new WebSocket(url)
      this.websock.onopen = this.websocketOnopen
      this.websock.onerror = this.websocketOnerror
      this.websock.onmessage = this.websocketOnmessage
      this.websock.onclose = this.websocketOnclose
    },
    websocketOnopen() {
      console.log('WebSocket连接成功')
      this.heartCheck.start()
    },
    websocketOnerror(e) {
      console.log(e, 'WebSocket连接发生错误')
      this.reconnect()
    },
    ungzip(zipData, callback) {
      var unZipData = '' //解压缩后数据

      //在浏览器中，必须使用atob，并且您需要使用例如Uint8Array.from将解码的数据转换为Uint8Array

      var strData = atob(zipData)

      const gzipedDataArray = Uint8Array.from(strData, c => c.charCodeAt(0))

      const ungzipedData = pako.ungzip(gzipedDataArray)

      // console.log('ungziped data', new TextDecoder().decode(ungzipedData))
      // 在ungzip上的结果是一个Unit8Array。如果你想把它转换回一个字符串，你需要再次解码：

      unZipData = new TextDecoder().decode(ungzipedData)

      // 还原成对象
      if (unZipData) {
        unZipData = JSON.parse(unZipData)
      }
      //利用回调进行处理后续操作
      if (callback) {
        callback(unZipData)
      }
    },
    websocketOnmessage(e) {
      // console.log('-----接收消息-------', e.data)
      let data = JSON.parse(e.data) //解析对象   eval('(' + e.data + ')');
      //   console.log('-----接收消息-------', data.cmd, data.data)
      if (data.cmd == 'SCADA') {
        if (this.isCurrentWindow) {
          // 解压数据
          this.ungzip(data.data, changeData => {
            // console.log('-----接收消息-------', changeData)
            // 处理业务逻辑
            this.$bus.$emit('SCADAMsg', changeData)
          })
        }
      }
      // 场群协同
      if (data.cmd == 'CQKZ') {
        if (this.isCurrentWindow) {
          //   console.log('-----接收消息-------', data.data)
          this.$bus.$emit('CQKZMsg', data.data)
        }
      }
      // 无功功率
      if (data.cmd == 'DWWGKZ') {
        if (this.isCurrentWindow) {
          this.ungzip(data.data, changeData => {
            // 处理业务逻辑
            this.$bus.$emit('DWWGKZMsg', changeData)
          })
        }
      }

      if (data.cmd == 'dict') {
        // 重载字典
        this.getSysDict()
      }
      if (data.cmd == 'alarm') {
        // 告警刷新
        this.$bus.$emit('SCADAAlram')
      }
      this.heartCheck.reset()
    },
    websocketOnclose(e) {
      console.log('connection closed (' + e.code + ')')
      this.reconnect()
    },
    websocketSend(text) {
      // 数据发送
      try {
        this.websock.send(text)
      } catch (err) {
        console.log('send failed (' + err.code + ')')
      }
    },
    reconnect() {
      if (this.lockReconnect) return
      this.lockReconnect = true
      //没连接上会一直重连，设置延迟避免请求过多
      setTimeout(() => {
        console.info('尝试重连...')
        this.initWebSocket()
        this.lockReconnect = false
      }, 5000)
    },
    heartCheckFun() {
      var that = this
      //心跳检测,每10分钟心跳一次
      this.heartCheck = {
        timeout: 1000 * 60 * 10,
        timeoutObj: null,
        serverTimeoutObj: null,
        reset: function() {
          clearTimeout(this.timeoutObj)
          this.start()
          return this
        },
        start: function() {
          var self = this
          this.timeoutObj = setTimeout(function() {
            //这里发送一个心跳，后端收到后，返回一个心跳消息，
            //onmessage拿到返回的心跳就说明连接正常
            const account = JSON.parse(sessionStorage.getItem('currentUser'))
              .account
            that.websocketSend('{"messageType":"ping","message":"web"}')
            console.info('客户端发送心跳')
            //self.serverTimeoutObj = setTimeout(function(){//如果超过一定时间还没重置，说明后端主动断开了
            //  that.websock.close();//如果onclose会执行reconnect，我们执行ws.close()就行了.如果直接执行reconnect 会触发onclose导致重连两次
            //}, self.timeout)
          }, this.timeout)
        }
      }
    },
    //---------websocket 接收字典被修改的信息，请求新字典数据。-------------
    // websocket sendMessage()

    // websocketSend
    //  messageType: 类型 ping //心跳，topic //群发送，user //向目标用户发消息
    //  account :登录用户account的账号
    //  chatId :目标用户account的账号
    //  message :消息内容

    getSysDict() {
      portal.getSysDict().then(data => {
        console.log(data, '字典')
        if (data) {
          sessionStorage.setItem('dict_data', JSON.stringify(data))
        }
      })
    }
  }
}
</script>

<style lang="scss" scoped>
@import '@/assets/css/element-variables.scss';
.fold-left-enter-active {
  animation: fold-left-in 0.5s;
}
.fold-left-leave-active {
  animation: fold-left-on 0.5s;
}
@keyframes fold-left-in {
  0% {
    transform: translate3d(-10%, 0, 0);
    opacity: 0;
  }
  50% {
    opacity: 1;
  }
  100% {
    transform: translate3d(0, 0, 0);
    opacity: 1;
  }
}
@keyframes fold-left-on {
  0% {
    transform: translate3d(0%, 0, 0);
    opacity: 1;
  }
  50% {
    opacity: 0;
  }
  100% {
    transform: translate3d(10%, 0, 0);
    opacity: 0;
  }
}

.fold-right-enter-active {
  animation: fold-right-in 0.5s;
}
.fold-right-leave-active {
  animation: fold-right-on 0.5s;
}
@keyframes fold-right-in {
  0% {
    transform: translate3d(10%, 0, 0);
    opacity: 0;
  }
  50% {
    opacity: 1;
  }
  100% {
    transform: translate3d(0, 0, 0);
    opacity: 1;
  }
}
@keyframes fold-right-on {
  0% {
    transform: translate3d(0, 0, 0);
    opacity: 1;
  }
  50% {
    opacity: 0;
  }
  100% {
    transform: translate3d(-10%, 0, 0);
    opacity: 0;
  }
}
.main-container > .el-card {
  // box-shadow: 0px 0px 12px 0 rgba($color: #000000, $alpha: 0.1);
}
.home-page > .el-card {
  box-shadow: none;
}
.el-main > .el-card {
  border-radius: 0;
}

.app-box__card {
  height: 100%;
  background-color: $--background-color-base;
  border-color: $--background-color-base;
}

.app-box__card /deep/ > .el-card__header {
  padding: 12px 20px;
  margin-bottom: 10px;
}

.app-box__card /deep/ > .el-card__body {
  height: 100%;
  padding: 0;
}
.single-sepical.app-box__card /deep/ > .el-card__body {
  height: 100%;
}

.home-page > .app-box__card /deep/ > .el-card__body {
  height: 100% !important;
}

.box-card-noheader /deep/ > .el-card__body {
  height: 100% !important;
}

>>> .el-tree--highlight-current
  .el-tree-node.is-current
  > .el-tree-node__content {
  background-color: rgba(155, 194, 230, 0.2) !important;
}
>>> .el-breadcrumb__item:last-child .el-breadcrumb__inner {
  color: #211bce;
  font-weight: bold;
}
.total-layout {
  width: 100%;
  height: 100%;
  overflow: hidden;
  background: url(../../assets/img/bg2.png) center / 100% 100% no-repeat;
}
.body-container {
  width: 100%;
  height: 100%;
  overflow: hidden;
}
.main-container {
  overflow: hidden;
  width: 100%;
  height: 100%;
  padding: 10px;
  box-sizing: border-box;
}
</style>
